=== modified file 'bzrlib/plugins/weave_fmt/workingtree.py'
--- bzrlib/plugins/weave_fmt/workingtree.py	2011-08-30 09:29:03 +0000
+++ bzrlib/plugins/weave_fmt/workingtree.py	2011-08-30 17:32:29 +0000
@@ -17,25 +17,24 @@
 """Weave-era working tree objects."""
 
 from cStringIO import StringIO
-import errno
 
 from bzrlib import (
     conflicts as _mod_conflicts,
     errors,
-    hashcache,
     inventory,
     osutils,
     revision as _mod_revision,
-    trace,
     transform,
     xml5,
     )
 from bzrlib.decorators import needs_read_lock
 from bzrlib.transport.local import LocalTransport
 from bzrlib.workingtree import (
-    InventoryWorkingTree,
     WorkingTreeFormat,
     )
+from bzrlib.workingtree_3 import (
+    PreDirStateWorkingTree,
+    )
 
 
 def get_conflicted_stem(path):
@@ -138,7 +137,7 @@
         return wt
 
 
-class WorkingTree2(InventoryWorkingTree):
+class WorkingTree2(PreDirStateWorkingTree):
     """This is the Format 2 working tree.
 
     This was the first weave based working tree.
@@ -148,27 +147,6 @@
 
     def __init__(self, basedir, *args, **kwargs):
         super(WorkingTree2, self).__init__(basedir, *args, **kwargs)
-
-        # update the whole cache up front and write to disk if anything changed;
-        # in the future we might want to do this more selectively
-        # two possible ways offer themselves : in self._unlock, write the cache
-        # if needed, or, when the cache sees a change, append it to the hash
-        # cache file, and have the parser take the most recent entry for a
-        # given path only.
-        wt_trans = self.bzrdir.get_workingtree_transport(None)
-        cache_filename = wt_trans.local_abspath('stat-cache')
-        self._hashcache = hashcache.HashCache(basedir, cache_filename,
-            self.bzrdir._get_file_mode(),
-            self._content_filter_stack_provider())
-        hc = self._hashcache
-        hc.read()
-        # is this scan needed ? it makes things kinda slow.
-        #hc.scan()
-
-        if hc.needs_write:
-            trace.mutter("write hc")
-            hc.write()
-
         # WorkingTree2 has more of a constraint that self._inventory must
         # exist. Because this is an older format, we don't mind the overhead
         # caused by the extra computation here.
@@ -183,12 +161,6 @@
         return [('trees', self.last_revision())]
 
 
-    @needs_read_lock
-    def get_file_sha1(self, file_id, path=None, stat_value=None):
-        if not path:
-            path = self._inventory.id2path(file_id)
-        return self._hashcache.get_sha1(path, stat_value)
-
     def lock_tree_write(self):
         """See WorkingTree.lock_tree_write().
 
@@ -206,20 +178,6 @@
             self.branch.unlock()
             raise
 
-    def _write_hashcache_if_dirty(self):
-        """Write out the hashcache if it is dirty."""
-        if self._hashcache.needs_write:
-            try:
-                self._hashcache.write()
-            except OSError, e:
-                if e.errno not in (errno.EPERM, errno.EACCES):
-                    raise
-                # TODO: jam 20061219 Should this be a warning? A single line
-                #       warning might be sufficient to let the user know what
-                #       is going on.
-                trace.mutter('Could not write hashcache for %s\nError: %s',
-                              self._hashcache.cache_file_name(), e)
-
     def unlock(self):
         # we share control files:
         if self._control_files._lock_count == 3:

=== modified file 'bzrlib/workingtree_3.py'
--- bzrlib/workingtree_3.py	2011-08-30 09:29:03 +0000
+++ bzrlib/workingtree_3.py	2011-08-30 17:32:29 +0000
@@ -40,19 +40,11 @@
     WorkingTreeFormat,
     )
 
-class WorkingTree3(InventoryWorkingTree):
-    """This is the Format 3 working tree.
-
-    This differs from the base WorkingTree by:
-     - having its own file lock
-     - having its own last-revision property.
-
-    This is new in bzr 0.8
-    """
+
+class PreDirStateWorkingTree(InventoryWorkingTree):
 
     def __init__(self, basedir='.', *args, **kwargs):
-        super(WorkingTree3, self).__init__(basedir, *args, **kwargs)
-
+        super(PreDirStateWorkingTree, self).__init__(basedir, *args, **kwargs)
         # update the whole cache up front and write to disk if anything changed;
         # in the future we might want to do this more selectively
         # two possible ways offer themselves : in self._unlock, write the cache
@@ -73,6 +65,37 @@
             trace.mutter("write hc")
             hc.write()
 
+    def _write_hashcache_if_dirty(self):
+        """Write out the hashcache if it is dirty."""
+        if self._hashcache.needs_write:
+            try:
+                self._hashcache.write()
+            except OSError, e:
+                if e.errno not in (errno.EPERM, errno.EACCES):
+                    raise
+                # TODO: jam 20061219 Should this be a warning? A single line
+                #       warning might be sufficient to let the user know what
+                #       is going on.
+                trace.mutter('Could not write hashcache for %s\nError: %s',
+                              self._hashcache.cache_file_name(), e)
+
+    @needs_read_lock
+    def get_file_sha1(self, file_id, path=None, stat_value=None):
+        if not path:
+            path = self._inventory.id2path(file_id)
+        return self._hashcache.get_sha1(path, stat_value)
+
+
+class WorkingTree3(PreDirStateWorkingTree):
+    """This is the Format 3 working tree.
+
+    This differs from the base WorkingTree by:
+     - having its own file lock
+     - having its own last-revision property.
+
+    This is new in bzr 0.8
+    """
+
     @needs_read_lock
     def _last_revision(self):
         """See Mutable.last_revision."""
@@ -112,26 +135,6 @@
         finally:
             self.branch.unlock()
 
-    def _write_hashcache_if_dirty(self):
-        """Write out the hashcache if it is dirty."""
-        if self._hashcache.needs_write:
-            try:
-                self._hashcache.write()
-            except OSError, e:
-                if e.errno not in (errno.EPERM, errno.EACCES):
-                    raise
-                # TODO: jam 20061219 Should this be a warning? A single line
-                #       warning might be sufficient to let the user know what
-                #       is going on.
-                trace.mutter('Could not write hashcache for %s\nError: %s',
-                              self._hashcache.cache_file_name(), e)
-
-    @needs_read_lock
-    def get_file_sha1(self, file_id, path=None, stat_value=None):
-        if not path:
-            path = self._inventory.id2path(file_id)
-        return self._hashcache.get_sha1(path, stat_value)
-
 
 class WorkingTreeFormat3(WorkingTreeFormat):
     """The second working tree format updated to record a format marker.

